<?php

defined('ABSPATH') or die('No script kiddies please!');

/**
 * REST API Route List with functions...
 *
 * @package Swipecart
 * @author Manthan Kanani
 * @since 1.0.0
**/
class REST_API_v1_Controller {

	/**
	 * Setup path and define AuthCombo  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function __construct() {
		$this->namespace	= 'sc/v1';

		$auth_generation 			= new SC_AuthGeneration();
		$this->authCombo 			= $auth_generation->getAuth();
		$this->storeFrontToken 		= $this->authCombo['StoreFrontToken'];
	}

	/**
	 * Get Product by Categories  
	 *
	 * @author Manthan Kanani	
	 * @since 2.2.1
	**/
	public function getAllCollectionsCategories($request){
		$api 		= new APIClass();
		$order 		= $request->get_param('order');
		$orderBy 	= $request->get_param('orderBy');
		$page 		= $request->get_param('page');
		$pp 		= $request->get_param('pp');
		$search 	= $request->get_param('search');

		$result = $api->getAllCollectionsCategories($order, $orderBy, $page, $pp, $search);
		if($result) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "success", 
				"data" 		=> array("collections" => $result)
			);
		} else {
			$arr = array("success" => false, "message" => "Not any Collections found");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}

	/**
	 * Get Product by Categories  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function GetCollectionsByIds($request){
		$api 		= new APIClass();
		$id 		= $request->get_param('ids');

		$result = $api->GetCollectionsByIds($id, false);
		if($result) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "success", 
				"data" 		=> array("collections" => $result)
			);
		} else {
			$arr = array("success" => false, "message" => "Not any collections found");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}

	

	/**
	 * Get Customer Orders  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function getOrders($request){
		$api 		= new APIClass();
		$customer 	= $request->get_param('customer');
		$order 		= $request->get_param('order');
		$orderBy 	= $request->get_param('orderBy');
		$page 		= $request->get_param('page');
		$pp 		= $request->get_param('pp');

		$result = $api->getOrders($order, $orderBy, $page, $pp, $customer);
		if($result) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "success", 
				"data" 		=> array("order" => $result)
			);
		} else {
			$arr = array("status" => false, "error" => "No Data !!");
		}
		return $arr;
	}


	/**
	 * Get Order By ID  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function getOrderById($request){
		$api 		= new APIClass();

		$id 		= $request->get_param('id');

		$result = $api->getOrderByID($id, false);
		if($result) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "success", 
				"data" 		=> array("order" => $result)
			);
		} else {
			$arr = array("status" => false, "error" => "No Data !!");
		}
		return $arr;
	}


	/**
	 * Get All Products with search,category,tag filter  
	 *
	 * @author Manthan Kanani	
	 * @since 1.0.0
	**/
	public function getProducts($request){
		$api 			= new APIClass();
		$category 		= $request->get_param('category');
		$category 		= ($category)? $category : array() ;
		$order 			= $request->get_param('order');
		$orderBy 		= $request->get_param('orderBy');
		$page 			= $request->get_param('page');
		$pp 			= $request->get_param('pp');
		$search 		= $request->get_param('search');
		$filter 		= $request->get_param('filter');

		$result = $api->getProducts($order, $orderBy, $page, $pp, $category, $search, $filter);
		if($result) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "success", 
				"data" 		=> array("products" => $result)
			);
		} else {
			$arr = array("success" => false, "message" => "Not any products found");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}


	/**
	 * Search Product By ID  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function SearchProducts($request){
		$api 			= new APIClass();
		$page 			= $request->get_param('page');
		$pp 			= $request->get_param('pp');
		$search 		= $request->get_param('search');

		$result = $api->getProducts('', '', $page, $pp, array(), $search);
		if($result) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "success", 
				"data" 		=> array("products" => $result)
			);
		} else {
			$arr = array("success" => false, "message" => "Not any products found");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}


	/**
	 * Get Products by IDs  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function GetProductsByIds($request){
		$api 		= new APIClass();
		$id 		= $request->get_param('ids');

		$result = $api->GetProductsByIds($id, false);
		if($result) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "success", 
				"data" 		=> array("products" => $result)
			);
		} else {
			$arr = array("success" => false, "message" => "Not any products found");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}

	/**
	 * Get Single Product  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function GetProductById($request){
		$api 		= new APIClass();
		$id 		= $request->get_param('id');

		$result = $api->getProductByID($id, false);
		if($result) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "success", 
				"data" 		=> array("product" => $result)
			);
		} else {
			$arr = array("success" => false, "message" => "No product available");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}

	/**
	 * Get Products by Collections IDs  
	 *
	 * @author Manthan Kanani	
	 * @since 2.2.1
	**/
	public function getProductsByCollectionId($request){
		$api 			= new APIClass();
		$category 		= $request->get_param('id');
		$category 		= ($category)? $category : array() ;
		$order 			= $request->get_param('order');
		$orderBy 		= $request->get_param('orderBy');
		$page 			= $request->get_param('page');
		$pp 			= $request->get_param('pp');
		$filter 		= $request->get_param('filter');
		
		$result = $api->getProducts($order, $orderBy, $page, $pp, array($category), '', $filter);
		if($result) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "success", 
				"data" 		=> array("products" => $result)
			);
		} else {
			$arr = array("success" => false, "message" => "Not any products found");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}

	/**
	 * Get Products by Collections IDs  
	 *
	 * @author Manthan Kanani	
	 * @since 2.2.1
	**/
	public function getProductsByVarientId($request){
		$api 		= new APIClass();
		$ids 		= $request->get_param('ids');
		
		if($result = $api->GetProductsByVarientId($ids)) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "success", 
				"data" 		=> array("products" => $result)
			);
		} else {
			$arr = array("success" => false, "message" => "Not any products found");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}


	/**
	 * Get Product Filters  
	 *
	 * @author Manthan Kanani	
	 * @since 2.2.1
	**/
	public function getProductFilters($request){
		$api 		= new APIClass();
		
		if($result = $api->getProductFilters()) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "success", 
				"data" 		=> array("filter" => array( "sections" => $result ))
			);
		} else {
			$arr = array("success" => false, "message" => "Not any product filters found");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}


	/**
	 * generate Access Token by email and password  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function SignIn($request){
		$api 		= new APIClass();
		$email 		= $request->get_param('email');
		$password 	= $request->get_param('password');

		$result = $api->SignIn($email, $password);
		if($result) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "Welcome", 
				"data" 		=> $result
			);
		} else {
			$arr = array("success" => false, "message" => "Could not log in");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}

	/**
	 * register User  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function customerSignUp($request){
		$api 		= new APIClass();
		$email 		= $request->get_param('email');
		$first_name = $request->get_param('firstName');
		$last_name 	= $request->get_param('lastName');
		$phone 		= $request->get_param('phone');
		$password 	= $request->get_param('password');

		$result = $api->customerSignUp($email, $first_name, $last_name, $phone, $password);

		if($result) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "signed up", 
				"data" 		=> array(
					"customerCreate" => array(
						"customer" => $result
					)
				)
			);
		} else {
			$arr = array("success" => false, "message" => "could not finish sign up");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}

	/**
	 * Forgot password of registered User  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function forgotPassword($request){
		$api 		= new APIClass();
		$email 		= $request->get_param('email');

		$result = $api->forgotPassword($email);
		if($result) {
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "reset password link sent", 
				"data" 		=> $result
			);
		} else {
			$arr = array("success" => false, "message" => "could not send email link");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}


	/**
	 * Update User Profile  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.4
	**/
	public function ChangeProfile($request){
		$api 				= new APIClass();
		$authorization 		= $request->get_header('Authorization');
		$first_name 		= $request->get_param('firstName');
		$last_name 			= $request->get_param('lastName');
		$phone 				= $request->get_param('phone');

		$StoreFrontToken	= str_replace('Bearer ', '', $authorization);

		if($user = $api->tokenValidation($StoreFrontToken)){
			$userdata['ID'] = $user->ID;
			if($first_name) $userdata['first_name'] = $first_name;
			if($last_name) $userdata['last_name'] 	= $last_name;
			$userdata['billing_phone'] 	= $phone ?? null;
			
			if($result = $api->ChangeProfile($userdata)){
				$arr = array(
					"success" 	=> true, 
					"message" 	=> "Profile changes", 
					"data" 		=> $result
				);
			} else {
				$arr = array("success" => false, "message" => "User not updated");
				return new WP_REST_Response($arr, 400);
			}
		} else {
			$arr = array("success" => false, "message" => "Could not log in");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}


	/**
	 * Update User Profile  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.4
	**/
	public function ChangePassword($request){
		$api 				= new APIClass();
		$authorization 		= $request->get_header('Authorization');
		$user_pass 			= $request->get_param('password');

		$StoreFrontToken	= str_replace('Bearer ', '', $authorization);

		if($user = $api->tokenValidation($StoreFrontToken)){
			$userdata['ID'] = $user->ID;
			if($user_pass) $userdata['user_pass'] 	= $user_pass;
			
			if($result = $api->ChangeProfile($userdata)){
				$arr = array(
					"success" 	=> true, 
					"message" 	=> "Profile changes", 
					"data" 		=> $result
				);
			} else {
				$arr = array("success" => false, "message" => "Password not updated");
				return new WP_REST_Response($arr, 400);
			}
		} else {
			$arr = array("success" => false, "message" => "Could not log in");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}


	/**
	 * Get Addresses  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function getAddresses($request){
		$api 					= new APIClass();
		$authorization 			= $request->get_header('Authorization');
		$StoreFrontToken		= str_replace('Bearer ','',$authorization);
		if($user = $api->tokenValidation($StoreFrontToken)){
			if($result = $api->getAddresses($user->ID)){
				$arr = array(
					"success" 	=> true, 
					"message" 	=> "address got", 
					"data" 		=> array("address" => $result)
				);
			} else {
				$arr = array("success" => false, "message" => "unable to retrive user");
				return new WP_REST_Response($arr, 400);
			}
		} else {
			$arr = array("success" => false, "error" => "Invalid User !!");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}


	/**
	 * Get Addresses  
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function updateAddress($request){
		$api 							= new APIClass();
		$authorization 					= $request->get_header('Authorization');

		$addressdata['id'] 				=  $request->get_param('id');
		$addressdata['address_1'] 		=  $request->get_param('address1');
		$addressdata['address_2'] 		=  $request->get_param('address2');
		$addressdata['city'] 			=  $request->get_param('city');
		$addressdata['company'] 		=  $request->get_param('company');
		$addressdata['country'] 		=  $request->get_param('country');
		$addressdata['first_name'] 		=  $request->get_param('firstName');
		$addressdata['last_name'] 		=  $request->get_param('lastName');
		$addressdata['phone'] 			=  $request->get_param('phone');
		$addressdata['state'] 			=  $request->get_param('province');
		$addressdata['postcode'] 		=  $request->get_param('zip');


		$StoreFrontToken	= str_replace('Bearer ','',$authorization);
		if($user = $api->tokenValidation($StoreFrontToken)){
			if($result = $api->updateAddress($user->ID, $addressdata)){
				$arr = array(
					"success" 	=> true, 
					"message" 	=> "address updated", 
					"data" 		=> $result
				);
			} else {
				$arr = array("success" => false, "error" => "Address does not exist");
				return new WP_REST_Response($arr, 400);
			}
		} else {
			$arr = array("success" => false, "error" => "Invalid User !!");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}


	/**
	 * Delete user and wipe out his data  
	 *
	 * @author Manthan Kanani	
	 * @since 2.3.0
	**/
	public function deleteCustomer($request){
		$api 				= new APIClass();
		$authorization 		= $request->get_header('Authorization');

		$userLoginToken	= str_replace('Bearer ','', $authorization);
		if($user = $api->tokenValidation($userLoginToken)){
			if($result = $api->deleteUser($user->ID, 'customer')){
				$arr = array(
					"success" 	=> true, 
					"message" 	=> "Customer Removed Successfully.", 
					"data" 		=> array( "role" => "customer", "removed" => $result )
				);
			} else {
				$arr = array("success" => false, "error" => "You're not a customer. Please Contact Site Administrator");
				return new WP_REST_Response($arr, 400);
			}
		} else {
			$arr = array("success" => false, "error" => "Invalid User !!");
			return new WP_REST_Response($arr, 404);
		}
		return $arr;
	}



	/**
	 * Generate Checkout
	 *
	 * @author Manthan Kanani	
	 * @since 2.0.0
	**/
	public function generateCheckout($request){
		$api 					= new APIClass();
		$line_items 			= $request->get_param('lineItems');
		$authorization 			= $request->get_header('Authorization');
		$StoreFrontToken		= str_replace('Bearer ','',$authorization);
		
		$user = $api->tokenValidation($StoreFrontToken) ?? null ;
		if(!$user && $StoreFrontToken !== $this->storeFrontToken){
			$arr = array("success" => false, "error" => "Invalid User !!");
			return new WP_REST_Response($arr, 400);
		}

		if($result = $api->generateCheckout($line_items, $StoreFrontToken)){
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "checkout generated", 
				"data" 		=> $result
			);
		} else {
			$arr = array("success" => false, "error" => "Invalid User !!");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;		
	}


	/**
	 * webhook topic
	 *
	 * @author Manthan Kanani	
	 * @since 2.4.0
	**/
	public function webhookTopic($request){
		$api 				= new APIClass();
		$action 			= $request->get_param('action');
		$topic 				= $request->get_param('topic');
		$callbackURL 		= $request->get_param('callbackURL');
		$callbackURL 		= ($callbackURL)? $callbackURL : null ;
		
		if($result = $api->webhookCRUD($action, $topic, $callbackURL)){
			if(in_array($action, ['c','u'])){
				$arr = array(
					"success" 	=> true, 
					"message" 	=> "Webhook saved gracefully"
				);
			} else {
				$arr = array(
					"success" 	=> true, 
					"message" 	=> "Webhook actioned gracefully",
					"data" 		=> $result
				);
			}
		} else {
			$arr = array("success" => false, "error" => "Invalid webhook !!");
			return new WP_REST_Response($arr, 400);
		}
		return $arr;
	}



	/**
	 * get public sitedata
	 *
	 * @author Manthan Kanani	
	 * @since 2.4.0
	**/
	public function getPublicSiteData($request){
		$api 			= new APIClass();

		$siteData = $api->getPublicSiteData();
		$arr = array(
			"success" 	=> true, 
			"message" 	=> "Site Data retrived Successfully", 
			"data" 		=> $siteData
		);
		return $arr;
	}

	

	/**
	 * Verify Authorization and return sideData
	 *
	 * @author Manthan Kanani	
	 * @since 2.1.0
	 * @version 2.4.0
	**/
	public function verifyAuthorization($request){
		$api 			= new APIClass();

		$siteData = $api->getSiteData();
		$arr = array(
			"success" 	=> true, 
			"message" 	=> "Authorized Successfully", 
			"data" 		=> $siteData
		);
		return $arr;
	}



	/**
	 * Verify Nonce
	 *
	 * @author Manthan Kanani	
	 * @since 2.1.1
	 * @version 2.4.0
	**/
	public function verifyNonce($request){
		$api 			= new APIClass();
		$nonce 			= $request->get_param('nonce');

		if($result = $api->nonceValidation($nonce)){
			$arr = array(
				"success" 	=> true, 
				"message" 	=> "Nonce Validated"
			);
		} else {
			$arr = array("success" => false, "error" => "Invalid Nonce !!");
			return new WP_REST_Response($arr, 401);
		}
		return $arr;
	}


	


	









	/*************************************************************
	**************************************************************
	** double curly bracket {{field}} is required field  
	** single curly bracket {field} is not mendetory each time
	** Intitial url : /wp-json/
	**
	** @header StoreFrontToken
	** @author Manthan Kanani
	**/
	public function register_routes() {
		
		/**
		 * Get Product Categories of product
		 * sc/v1/getAllCollections
		 *
		 * @param (str) {order}		=> ('asc'||'desc')
		 * @param (str) {orderBy}	=> ('id'||'name'||'popularity'||'date')
		 * @param (int) {page}		=> ({}>0)
		 * @param (int) {pp}		=> ({}>=0)
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'getAllCollections', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'getAllCollectionsCategories'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args' => array(
				'order' => array(
					'required' 			=> false,
					'sanitize_callback'	=> array($this, 'orderSanitizeCallback'),
					'validate_callback'	=> array($this, 'orderValidateCallback'),
				),
				'orderBy' => array(
					'required' 			=> false,
					'sanitize_callback'	=> array($this, 'orderBySanitizeCallback'),
					'validate_callback'	=> array($this, 'orderByValidateCallback'),
				),
				'page' => array(
					'default' 			=> 1,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> array($this, 'numericPositiveValidateCallback'),
				),
				'pp' => array(
					'default' 			=> 10,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> 'rest_is_integer',
				),
				'search' => array(
					'required' 			=> false,
					'sanitize_callback'	=> 'sanitize_text_field',
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				)
			)
		]);


		/**
		 * Get Products by IDs
		 * sc/v1/GetCollectionsById
		 *
		 * @param (int) {{ids}}	=> (array({})>0)
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'GetCollectionsById', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'GetCollectionsByIds'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args' => array(
				'ids' => array(
					'required' 			=> true,
					'sanitize_callback'	=> array($this, 'numericPositiveArraySanitizeCallback'),
					'validate_callback'	=> array($this, 'numericPositiveArrayValidateCallback'),
				)
			)
		]);


		/**
		 * Get Product by ID
		 * sc/v1/GetProductById
		 *
		 * @param (int) {{id}}	=> ({}>0)
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'GetProductById', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'GetProductById'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args' => array(
				'id' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> array($this, 'numericPositiveValidateCallback'),
				)
			)
		]);


		/**
		 * Get Products by IDs
		 * sc/v1/GetProductsById
		 *
		 * @param (int) {{ids}}	=> (array({})>0)
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'GetProductsById', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'GetProductsByIds'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args' => array(
				'ids' => array(
					'required' 			=> true,
					'sanitize_callback'	=> array($this, 'numericPositiveArraySanitizeCallback'),
					'validate_callback'	=> array($this, 'numericPositiveArrayValidateCallback'),
				)
			)
		]);


		/**
		 * Search Products by Name
		 * sc/v1/SearchProducts
		 *
		 * @param (int) {{search}}	=> (!='')
		 * @param (int) {page}		=> ({}>0)
		 * @param (int) {pp}		=> ({}>=0)
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'SearchProducts', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'SearchProducts'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args' => array(
				'search' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'sanitize_text_field',
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				),
				'page' => array(
					'default' 			=> 1,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> array($this, 'numericPositiveValidateCallback'),
				),
				'pp' => array(
					'default' 			=> 10,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> 'rest_is_integer',
				),
			)
		]);



		/**
		 * get filtered Products 
		 * sc/v1/GetProducts
		 *
		 * @param (int) {{search}}	=> (!='')
		 * @param (int) {order}		=> ('asc'||'desc')
		 * @param (int) {orderBy}	=> ('id'||'best_selling'||'price'||'recommended'||'date')
		 * @param (int) {page}		=> ({}>0)
		 * @param (int) {pp}		=> ({}>=0)
		 * @param (int) {search}	=> (!='')
		 * @param (int) {filter}	=> ([{[sectionValue],[tags][{value}]}])
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'GetProducts', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'getProducts'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args' => array(
				'order' => array(
					'default' 			=> 'desc',
					'sanitize_callback'	=> array($this, 'orderSanitizeCallback'),
					'validate_callback'	=> array($this, 'orderValidateCallback'),
				),
				'orderBy' => array(
					'default' 			=> 'id',
					'sanitize_callback'	=> array($this, 'orderBySanitizeCallback'),
					'validate_callback'	=> array($this, 'orderByValidateCallback'),
				),
				'page' => array(
					'default' 			=> 1,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> array($this, 'numericPositiveValidateCallback'),
				),
				'pp' => array(
					'default' 			=> 10,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> 'rest_is_integer',
				),
				'category' => array(
					'required' 			=> false,
					'sanitize_callback'	=> array($this, 'numericPositiveArraySanitizeCallback'),
					'validate_callback'	=> array($this, 'numericPositiveArrayValidateCallback')
				),
				'filter' => array(
					'required' 			=> false,
					'sanitize_callback'	=> array($this, 'filterArraySanitizeCallback'),
					'validate_callback'	=> array($this, 'filterArrayValidateCallback'),
				),
				'search' => array('required' => false )
			)
		]);


		/**
		 * Search Products by Name
		 * sc/v1/GetProductsByCollectionId
		 *
		 * @param (int) {{id}}		=> (!='')
		 * @param (int) {order}		=> ('asc'||'desc')
		 * @param (int) {orderBy}	=> ('id'||'best_selling'||'price'||'recommended'||'date')
		 * @param (int) {page}		=> ({}>0)
		 * @param (int) {pp}		=> ({}>=0)
		 * @param (int) {search}	=> (!='')
		 * @param (int) {filter}	=> ([{[sectionValue],[tags][{value}]}])
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'GetProductsByCollectionId', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'getProductsByCollectionId'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args' => array(
				'id' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> array($this, 'numericPositiveValidateCallback'),
				),
				'order' => array(
					'default' 			=> 'desc',
					'sanitize_callback'	=> array($this, 'orderSanitizeCallback'),
					'validate_callback'	=> array($this, 'orderValidateCallback'),
				),
				'orderBy' => array(
					'default' 			=> 'id',
					'sanitize_callback'	=> array($this, 'orderBySanitizeCallback'),
					'validate_callback'	=> array($this, 'orderByValidateCallback'),
				),
				'page' => array(
					'default' 			=> 1,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> array($this, 'numericPositiveValidateCallback'),
				),
				'pp' => array(
					'default' 			=> 10,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> 'rest_is_integer',
				),
				'filter' => array(
					'required' 			=> false,
					'sanitize_callback'	=> array($this, 'filterArraySanitizeCallback'),
					'validate_callback'	=> array($this, 'filterArrayValidateCallback'),
				),
				'search' => array('required' => false )
			)
		]);


		/**
		 * Search Products by Name
		 * sc/v1/GetProductsByVarientId
		 *
		 * @param (int) {{ids}}		=> (!=formatted_array)
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'GetProductsByVarientId', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'getProductsByVarientId'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args' => array(
				'ids' => array(
					'required' 			=> true,
					'sanitize_callback'	=> array($this, 'cartVariationSanitizeCallback'),
					'validate_callback'	=> array($this, 'cartVariationValidateCallback'),
				)
			)
		]);


		/**
		 * Search Product Filters
		 * sc/v1/GetProductFilters
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'GetProductFilters', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'getProductFilters'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' )
		]);


		/**
		 * Get Product by ID
		 * sc/v1/GetOrderById
		 *
		 * @param (int) {id}	=> ({}>0)
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'GetOrderById', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'getOrderById'),
			'permission_callback' 	=> array( $this, 'verifyAccessTokenPermissionCallback' ),
			'args' => array(
				'id' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'sanitize_text_field',
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				)
			)
		]);


		/**
		 * Get Woocommerce Customer Orders
		 * sc/v1/GetOrders
		 *
		 * @param (int) {customer}	=> ({}>0)
		 * @param (int) {order}		=> ('asc'||'desc')
		 * @param (int) {orderBy}	=> ('id'||'name'||'popularity'||'date')
		 * @param (int) {page}		=> ({}>0)
		 * @param (int) {pp}		=> ({}>=0)
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'GetOrders', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'getOrders'),
			'permission_callback' 	=> array( $this, 'verifyAccessTokenPermissionCallback' ),
			'args' => array(
				'customer' => array(
					'default' 			=> 0,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> 'rest_is_integer',
				),
				'order' => array(
					'required' 			=> false,
					'sanitize_callback'	=> array($this, 'orderSanitizeCallback'),
					'validate_callback'	=> array($this, 'orderValidateCallback'),
				),
				'orderBy' => array(
					'required' 			=> false,
					'sanitize_callback'	=> array($this, 'orderBySanitizeCallback'),
					'validate_callback'	=> array($this, 'orderByValidateCallback'),
				),
				'page' => array(
					'default' 			=> 1,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> array($this, 'numericPositiveValidateCallback'),
				),
				'pp' => array(
					'default' 			=> 10,
					'sanitize_callback'	=> 'absint',
					'validate_callback'	=> 'rest_is_integer',
				)
			)
		]);

		
		/**
		 * Create Customer for Woocommerce
		 * sc/v1/SignUp
		 *
		 * @param (str) {{email}}		=> (!='')
		 * @param (str) {{firstName}}	=> (!='')
		 * @param (str) {{lastName}}	=> (!='')
		 * @param (str) {{password}}	=> (!='')
		 * @param (str) {phone}			=> (!='')
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'SignUp', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'customerSignUp'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args' => array(
				'email' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'sanitize_email',
					'validate_callback'	=> 'is_email',
				),
				'firstName' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'sanitize_text_field',
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				),
				'lastName' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'sanitize_text_field',
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				),
				'phone' => array(
					'required' 			=> false,
					'sanitize_callback'	=> array($this, 'telSanitizeCallback'),
					'validate_callback'	=> array($this, 'telValidateCallback'),
				),
				'password' => array(
					'required' 			=> true,
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				)
			)
		]);


		/**
		 * Get User token after Successfully loggedin
		 * sc/v1/SignIn
		 *
		 * @param (int) {{email}}		=> (!='')
		 * @param (int) {{password}}	=> (!='')
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'SignIn', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'SignIn'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args' => array(
				'email' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'sanitize_email',
					'validate_callback'	=> 'is_email',
				),
				'password' => array(
					'required' 			=> true,
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				)
			)
		]);


		/**
		 * Delete user as per UK norms
		 * sc/v1/DeleteUser
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'DeleteUser', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'deleteCustomer'),
			'permission_callback' 	=> array( $this, 'verifyAccessTokenPermissionCallback' ),
		]);


		/**
		 * Forgot User Password
		 * sc/v1/ForgotPassword
		 *
		 * @param (int) {{email}}		=> (!='')
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'ForgotPassword', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'forgotPassword'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args' => array(
				'email' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'sanitize_email',
					'validate_callback'	=> 'is_email',
				)
			)
		]);


		/**
		 * Create Customer for Woocommerce
		 * sc/v1/ChangeProfile
		 *
		 * @param (str) {{firstName}}	=> (!='')
		 * @param (str) {{lastName}}	=> (!='')
		 * @param (str) {phone}			=> (!='')
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'ChangeProfile', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'ChangeProfile'),
			'permission_callback' 	=> array( $this, 'verifyAccessTokenPermissionCallback' ),
			'args' => array(
				'firstName' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'sanitize_text_field',
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				),
				'lastName' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'sanitize_text_field',
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				),
				'phone' => array(
					'required' 			=> false,
					'sanitize_callback'	=> array($this, 'telSanitizeCallback'),
					'validate_callback'	=> array($this, 'telValidateCallback'),
				),
			)
		]);


		/**
		 * Change Password of User
		 * sc/v1/ChangePassword
		 *
		 * @param (str) {{password}}	=> (!='')
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'ChangePassword', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'ChangePassword'),
			'permission_callback' 	=> array( $this, 'verifyAccessTokenPermissionCallback' ),
			'args' => array(
				'password' => array(
					'required' 			=> true,
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				),
			)
		]);


		/**
		 * get Addresses
		 * sc/v1/GetAddresses
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'GetAddresses', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'getAddresses'),
			'permission_callback' 	=> array( $this, 'verifyAccessTokenPermissionCallback' ),
		]);


		/**
		 * update Address
		 * sc/v1/UpdateAddress
		 *
		 * @param (str) {{id}}			=> (!='')
		 * @param (str) {address1}		=> (!='')
		 * @param (str) {address2}		=> (!='')
		 * @param (str) {city}			=> (!='')
		 * @param (str) {company}		=> (!='')
		 * @param (str) {country}		=> (!='')
		 * @param (str) {firstName}		=> (!='')
		 * @param (str) {lastName}		=> (!='')
		 * @param (str) {phone}			=> (!='')
		 * @param (str) {province}		=> (!='')
		 * @param (str) {zip}			=> (!='')
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'UpdateAddress', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'updateAddress'),
			'permission_callback' 	=> array( $this, 'verifyAccessTokenPermissionCallback' ),
			'args' => array(
				'id' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'sanitize_text_field',
					'validate_callback'	=> array($this, 'addressIDValidateCallback'),
				),
				'address1' => array(
					'required' 			=> false,
					'sanitize_callback'	=> 'sanitize_text_field',
				),
				'address2' => array(
					'required' 			=> false,
					'sanitize_callback'	=> 'sanitize_text_field',
				),
				'city' => array(
					'required' 			=> false,
					'sanitize_callback'	=> 'sanitize_text_field',
				),
				'company' => array(
					'required' 			=> false,
					'sanitize_callback'	=> 'sanitize_text_field',
				),
				'country' => array(
					'required' 			=> false,
					'sanitize_callback'	=> 'sanitize_text_field',
				),
				'firstName' => array(
					'required' 			=> false,
					'sanitize_callback'	=> 'sanitize_text_field',
				),
				'lastName' => array(
					'required' 			=> false,
					'sanitize_callback'	=> 'sanitize_text_field',
				),
				'phone' => array(
					'required' 			=> false,
					'sanitize_callback'	=> array($this, 'telSanitizeCallback'),
					'validate_callback'	=> array($this, 'telValidateCallback'),
				),
				'province' => array(
					'required' 			=> false,
					'sanitize_callback'	=> 'sanitize_text_field',
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				),
				'zip' => array(
					'required' 			=> false,
					'sanitize_callback'	=> 'sanitize_text_field',
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				)
			)
		]);


		/**
		 * Generate Checkout URL
		 * sc/v1/GenerateCheckout
		 *
		 * @param (str) {{lineItems}}		=> (array())
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'GenerateCheckout', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'generateCheckout'),
			'permission_callback' 	=> '__return_true',
			'args'					=> array(
				'lineItems' => array(
					'required' 			=> true,
					'sanitize_callback'	=> array($this, 'lineItemsArraySanitizeCallback'),
					'validate_callback'	=> array($this, 'lineItemsArrayValidateCallback'),
				)
			)
		]);


		/**
		 * Test Authorization
		 * sc/v1/webhooks/woocommerce
		 *
		 * @param (str) {{action}}		=> (c,r,u,d)
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'webhooks/woocommerce', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'webhookTopic'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args'					=> array(
				'action' => array(
					'required' 			=> true,
					'sanitize_callback'	=> array($this, 'actionSanitizeCallback'),
					'validate_callback'	=> array($this, 'actionValidateCallback'),
				),
				'topic' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'sanitize_text_field',
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				),
				'callbackURL' => array(
					'required' 			=> false,
					'sanitize_callback'	=> 'sanitize_url',
					'validate_callback'	=> array($this, 'urlValidateCallback'),
				),
			)
		]);


		/**
		 * get Private Data
		 * sc/v1/GetSiteData
		 *
		 * @param NULL
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'GetSiteData', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'getPublicSiteData'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
		]);

		/**
		 * Test Authorization by secret
		 * sc/v1/verify/authorization
		 *
		 * @param NULL
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'verify/authorization', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'verifyAuthorization'),
			'permission_callback' 	=> array( $this, 'verifyHeaderSecretPermissionCallback' ),
		]);


		/**
		 * Verify Nonce
		 * sc/v1/verify/nonce
		 *
		 * @param NULL
		 *
		 * @return json
		 */
		register_rest_route($this->namespace, 'verify/nonce', [
			'methods' 				=> 'POST',
			'callback' 				=> array( $this, 'verifyNonce'),
			'permission_callback' 	=> array( $this, 'verifyHeaderPermissionCallback' ),
			'args'					=> array(
				'nonce' => array(
					'required' 			=> true,
					'sanitize_callback'	=> 'sanitize_text_field',
					'validate_callback'	=> array($this, 'textRequiredValidateCallback'),
				)
			)
		]);
	}








	/**
	 * Verify request Header Callbacks
	 *
	 * @return bool
	**/
	public function verifyHeaderPermissionCallback(WP_REST_Request $request){
		$api 						= new APIClass();
		$authorizationBearer  		= $request->get_header('Authorization');
		$BearerToken				= str_replace('Bearer ','',$authorizationBearer);

		if($BearerToken == $this->storeFrontToken) return true;
		if($api->tokenValidation($BearerToken)) return true;
		return false;
	}
	public function verifyAccessTokenPermissionCallback(WP_REST_Request $request){
		$api 					= new APIClass();
		$authorization 			= $request->get_header('Authorization');
		$BearerToken			= str_replace('Bearer ','',$authorization);

		if($api->tokenValidation($BearerToken)) return true;
		return false;
	}
	public function verifyHeaderSecretPermissionCallback(WP_REST_Request $request){
		$webhook 				= new WC_SC_Webhook();

		$authorization 			= $request->get_header('Authorization');
		$wooToken				= str_replace('Bearer ','',$authorization);

		if($webhook->isValidSecret($wooToken)) return true;
		return false;
	}

	/**
	 * Sanitize Callbacks
	 *
	 * @return filtered Value
	**/
	public function returnCSVArraySanitizeCallback($param){
		$param = explode(',', $param);
		$param = array_map('intval', $param);
		return $param;
	}
	public function numericPositiveArraySanitizeCallback($param){
		$result = array_map(function($v) {
			return $this->numericPositiveValidateCallback($v)? $v : '' ;
		}, $param);
		$result = array_filter($result);
		$result = array_unique($result);
		return array_values($result);
	}
	public function lineItemsArraySanitizeCallback($param){
		$result = array_map(function($v) {
			return array(
				'variation_id' 	=> (int) $v['varient_id'],
				'quantity' 		=> (int) $v['quantity']
			);
		}, $param);
		return $result;
	}
	public function filterArraySanitizeCallback($param){
		$result = array_map(function($v){
			$tags = array();
			if(!is_array($v['tags']) || !count($v['tags'])) return;
			foreach($v['tags'] as $tag){
				$tags[] = $tag['value'];
			}
			return array(
				'attribute' => $v['sectionValue'],
				'terms' 	=> $tags
			);
		}, $param);
		$result = array_filter($result);
		return array_values($result);
	}
	public function orderSanitizeCallback($param){
		return (in_array($param, array('desc', 'asc')))? $param : 'desc' ;
	}
	public function orderBySanitizeCallback($param){
		return (in_array($param, array('id','best_selling','price','recommended','name','popularity','date')))? $param : 'id';
	}
	public function telSanitizeCallback($param){
		$param = str_replace(' ', '', $param);
		$param = str_replace('-', '', $param);
		return $param;
	}
	public function actionSanitizeCallback($param){
		return (in_array($param, array('c','r','u','d')))? $param : 'r' ;
	}
	public function cartVariationSanitizeCallback($params){
		$result = array_map(function($v) {
			return array(
				'product_id' 	=> (int) $v['product_id'],
				'varient_id' 	=> (int) $v['varient_id'],
			);
		}, $params);
		return $result;
	}

	/**
	 * Validate Callbacks
	 *
	 * @return bool
	**/
	public function numericPositiveValidateCallback($param){
		return (is_numeric($param) && ($param>0));
	}
	public function numericPositiveArrayValidateCallback($param){
		return (is_array($param));
	}
	public function numericCSVValidateCallback($params){
		$param = explode(',', $params);
		foreach($param as $a) {
			if (!is_numeric($a)) return false;
		}
		return true;
	}
	public function cartVariationValidateCallback($params){
		foreach($params as $param){
			if(!is_numeric($param['product_id']) || !is_numeric($param['varient_id'])) return false;
		}
		return true;
	}
	public function lineItemsArrayValidateCallback($param){
		return (is_array($param));
	}
	public function filterArrayValidateCallback($param){
		return (is_array($param));
	}
	public function textRequiredValidateCallback($param){
		return (isset($param) && !empty($param));
	}
	public function telValidateCallback($param){
		$valid = true;
		if(empty($param)) return $valid;
		$param = $this->telSanitizeCallback($param);

		$mobileDigitsLength = strlen($param);
		if ($mobileDigitsLength < 10 || $mobileDigitsLength > 15) {
			$valid = false;
		} else {
			if (!preg_match("/^[+]?[1-9][0-9]{9,14}$/", $param)) {
				$valid = false;
			}
		}
		return $valid;
	}
	public function addressIDValidateCallback($param){
		$address = array('billing','shipping');
		$param = strtolower($param);
		return in_array($param, $address);
	}
	public function ratingMaxFiveValidateCallback($param){
		return (is_numeric($param) && ($param>=0 && $param<=5));
	}
	public function orderValidateCallback($param){
		return in_array($param, ['desc', 'asc']);
	}
	public function actionValidateCallback($param){
		return in_array($param, ['c','r','u','d']);
	}
	public function urlValidateCallback($param){
		return sanitize_url($param) ? true : false;
	}
	public function orderByValidateCallback($param){
		$orders = array('id','best_selling','price','recommended','name','popularity','date');
		$param = strtolower($param);
		return in_array($param, $orders);
	}
}

/**
 * Initialize Rest API Route  
 *
 * @author Manthan Kanani	
 * @since 1.0.0
**/
add_action('rest_api_init', function(){
	$controller = new REST_API_v1_Controller();
	$controller->register_routes();
});
